package cn.estsh.i3plus.core.apiservice.configuration;

import cn.estsh.i3plus.core.api.iservice.base.ICoreDataSourceService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.CoreDataSource;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.boot.web.servlet.context.AnnotationConfigServletWebServerApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2022/1/7 11:41
 * @Modify:
 **/
@Component
@Slf4j
public class CoreJdbcTemplateConfig {
    public static final String IOC_CORE_DATASOURCE_PREFIX = "CORE_DATA_SOURCE_";

    public static final String CORE_CACHE_POOL_PREFIX = "CORE_DATA_SOURCE:";

    /**
     * 最小活动连接数
     */
    private static final Integer MININUM_IDLE = 10;
    /**
     * 连接池大小
     */
    private static final Integer MAXIMUN_POOL_SIZE = 100;
    /**
     * max-lifetime 连接最大存活时间
     */
    private static final Integer MAX_LIFT_TIME = 200000;
    /**
     * borrow-connection 获取连接失败重新获等待最大时间，在这个时间内如果有可用连接，将返回
     */
    private static final Integer BORROW_CONNECTION_TIMEOUT = 10000;
    /**
     * 连接最大空闲时间
     */
    private static final Integer MAX_IDLE_TIME = 10000;

    private static final ConcurrentHashMap<Long, CoreDataSource> CACHE_SOURCE_MAP = new ConcurrentHashMap<>();

    @Autowired
    private AnnotationConfigServletWebServerApplicationContext context;


    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redis;

    @Autowired
    private ICoreDataSourceService datasourceService;

    public void setSpringIocJdbcTemplate(CoreDataSource coreDatasource) {

        try {
            if (CACHE_SOURCE_MAP.containsKey(coreDatasource.getId())) {
                log.info("remove the older coreDataSource:{}", coreDatasource);
                removeJdbcTemplateBean(coreDatasource);
            }
        } catch (Exception e) {
            log.error("Error occurred when removing the older coreDataSource", e);
        }

        log.info("Spring Ioc Init coreDataSource to Ioc:{}", coreDatasource);
        NamedParameterJdbcTemplate namedParameterJdbcTemplate = getNamedParameterJdbcTemplate(coreDatasource);

        BeanDefinitionBuilder beanDefinitionBuilder = BeanDefinitionBuilder.genericBeanDefinition(CoreJdbcTemplate.class);
        beanDefinitionBuilder.addPropertyValue("source", coreDatasource);
        beanDefinitionBuilder.addPropertyValue("coreJdbcTemplate", namedParameterJdbcTemplate);

        //存入redis
        redis.putObject(CORE_CACHE_POOL_PREFIX + coreDatasource.getId(), coreDatasource);

        //手动注册bean
        context.getDefaultListableBeanFactory().registerBeanDefinition(IOC_CORE_DATASOURCE_PREFIX + coreDatasource.getId(), beanDefinitionBuilder.getBeanDefinition());
        CACHE_SOURCE_MAP.put(coreDatasource.getId(), coreDatasource);
        log.info(" Spring IOC Init Bean WMSJdbcTemplate.class Successful key:{}", IOC_CORE_DATASOURCE_PREFIX + coreDatasource.getId());

    }

    /**
     * 获取core的数据源链接，根据与redis中的缓存做对比，是否修改了数据源
     *
     * @param key
     * @return
     */
    public CoreJdbcTemplate getCoreJdbcTemplate(Long key) {

        try {
            CoreDataSource cacheDatasource = (CoreDataSource) redis.getObject(CORE_CACHE_POOL_PREFIX + key);
            if (Objects.isNull(cacheDatasource)) {
                return (CoreJdbcTemplate) context.getBean(IOC_CORE_DATASOURCE_PREFIX + key);
            }
            CoreDataSource coreDatasource = datasourceService.getCoreDatasourceObject(key);
            if (!cacheDatasource.equals(coreDatasource)) {
                resetJdbcTemplateBean(coreDatasource);
            }
            return (CoreJdbcTemplate) context.getBean(IOC_CORE_DATASOURCE_PREFIX + key);
        } catch (BeansException e) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("【数据源】数据库 [" + key + "]数据源不在系统控制当中")
                    .setErrorSolution("请检查数据源配置信息")
                    .build();
        }

    }

    /**
     * 移除数据源，关闭连接池，从容器中删除bean
     *
     * @param coreDatasource
     */
    public void removeJdbcTemplateBean(CoreDataSource coreDatasource) {
        Object bean;
        try {
            bean = context.getBean(IOC_CORE_DATASOURCE_PREFIX + coreDatasource.getId());
        } catch (BeansException e) {
            log.error("无法获取到该数据源的Bean {}", coreDatasource.getId());
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode())
                    .setErrorDetail("无法获取该数据源的bean").build();
        }
        //关闭连接池
        ((HikariDataSource) ((CoreJdbcTemplate) bean).getNamedParameterJdbcTemplate().getJdbcTemplate().getDataSource()).close();
        //销毁bean
        context.getDefaultListableBeanFactory().removeBeanDefinition(IOC_CORE_DATASOURCE_PREFIX + coreDatasource.getId());
        redis.deleteKey(CORE_CACHE_POOL_PREFIX + coreDatasource.getId());
    }

    /**
     * 封装NamedParameterJdbcTemplate类
     *
     * @param
     * @return
     */
    private NamedParameterJdbcTemplate getNamedParameterJdbcTemplate(CoreDataSource coreDataSource) {
        log.info("【数据源】创建数据源coreDataSource：{}", coreDataSource);
        CommonEnumUtil.DATA_SOURCE_TYPE type = CommonEnumUtil.DATA_SOURCE_TYPE.valueOf(coreDataSource.getSourceType());

        DataSourceBuilder<?> dataSourceBuilder = DataSourceBuilder.create();
        dataSourceBuilder.type(HikariDataSource.class);
        dataSourceBuilder.driverClassName(type.getDriverClassName());
        dataSourceBuilder.url(type.getJDBCUrl(coreDataSource.getSourceDataBaseName(), coreDataSource.getSourceHost(), coreDataSource.getSourcePort()));
        dataSourceBuilder.username(coreDataSource.getSourceUserName());
        dataSourceBuilder.password(coreDataSource.getSourcePassword());

        HikariDataSource dataSource = (HikariDataSource) dataSourceBuilder.build();
        //连接池大小
        dataSource.setMaximumPoolSize(MAXIMUN_POOL_SIZE);
        dataSource.setMinimumIdle(MININUM_IDLE);
        return new NamedParameterJdbcTemplate(dataSource);

    }

    /**
     * 检查是否能连接
     *
     * @param coreDataSource
     * @return
     */
    public boolean checkConnection(CoreDataSource coreDataSource) {
        NamedParameterJdbcTemplate template = getNamedParameterJdbcTemplate(coreDataSource);
        try {
            Connection connection = Objects.requireNonNull(template.getJdbcTemplate().getDataSource()).getConnection();
            boolean isConn = connection != null;
            if (isConn) {
                if (!connection.isClosed()) {
                    connection.close();
                }
            }
            setSpringIocJdbcTemplate(coreDataSource);
            return isConn;
        } catch (SQLException e) {
            log.error("【数据源】 数据源连接检查失败 Error Message:{}", e.getMessage());
        }
        return false;
    }


    private void resetJdbcTemplateBean(CoreDataSource coreDataSource) {
        removeJdbcTemplateBean(coreDataSource);
        setSpringIocJdbcTemplate(coreDataSource);
    }
}
