package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsTemplateService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsTemplate;
import cn.estsh.i3plus.sdk.sms.base.SmsUtilFactory;
import cn.estsh.i3plus.sdk.sms.base.pojo.SmsResult;
import cn.estsh.impp.framework.base.controller.BaseController;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Description : 短信模板
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-12-14 20:21
 * @Modify:
 **/
@RestController
@Api(tags = "短信模板")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-sms-template")
public class SysSmsTemplateController extends CrudBaseController<SysSmsTemplate> {

    private static final Logger LOGGER = LoggerFactory.getLogger(BaseController.class);

    @Autowired
    private ISysSmsTemplateService sysSmsTemplateService;

    /**
     * 新增模板信息
     *
     * @param bean 待新增数据
     * @return 操作结果
     */
    @Override
    public ResultBean insert(SysSmsTemplate bean) {
        try {
            ValidatorBean.beginValid(bean)
                    .notNull("templateCode", bean.getTemplateCode())
                    .notNull("templateType", bean.getTemplateType())
                    .notNull("templateName", bean.getTemplateName())
                    .notNull("templateContent", bean.getTemplateContent());

            bean.setTemplateStatus(ImppEnumUtil.SMS_TEMPLATE_STATUS.UNDER_REVIEW.getValue());

            // 唯一校验
            if (!sysSmsTemplateService.checkSysSmsTemplateOnly(bean)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                        .setErrorDetail("模板代码已存在")
                        .setErrorSolution("请重新操作")
                        .build();
            }

            SmsResult<SysSmsTemplate> smsResult = SmsUtilFactory.getSmsTemplateUtil().insertSmsTemplate(bean);
            if (!smsResult.isSuccess()) {
                LOGGER.error("【SMS】 短信模板保存失败！{}", smsResult);
                return ResultBean.fail(smsResult.getErrMsg()).setResultObject(smsResult);
            }

            bean = getCrudService().insert(bean);

            return ResultBean.success("操作成功")
                    .setResultObject(bean)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改模板信息
     *
     * @param bean 待更新数据
     * @return 操作结果
     */
    @Override
    public ResultBean update(SysSmsTemplate bean) {
        try {
            ValidatorBean.beginValid(bean)
                    .notNull("id", bean.getId())
                    .notNull("templateCode", bean.getTemplateCode())
                    .notNull("templateType", bean.getTemplateType())
                    .notNull("templateName", bean.getTemplateName())
                    .notNull("templateContent", bean.getTemplateContent());

            // 唯一校验
            if (!sysSmsTemplateService.checkSysSmsTemplateOnly(bean)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                        .setErrorDetail("模板代码已存在")
                        .setErrorSolution("请重新操作")
                        .build();
            }

            if (CommonEnumUtil.TRUE_OR_FALSE.valueOfBoolean(bean.getIsModifyContent())) {
                SmsResult<SysSmsTemplate> smsResult = SmsUtilFactory.getSmsTemplateUtil().updateSmsTemplate(bean);
                if (!smsResult.isSuccess()) {
                    LOGGER.error("【SMS】 短信模板保存失败！{}", smsResult);
                    return ResultBean.fail(smsResult.getErrMsg()).setResultObject(smsResult);
                }
            }

            bean = getCrudService().update(bean);
            return ResultBean.success("操作成功")
                    .setResultObject(bean)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id删除短信模板
     *
     * @param id 操作id
     * @return 操作结果
     */
    @Override
    public ResultBean delete(Long id) {
        try {
            ValidatorBean.checkNotNull(id, "请选需要删除短信模板！");
            SysSmsTemplate smsTemplate = getCrudService().get(id);
            ValidatorBean.checkNotNull(smsTemplate, "短信模板不存在！");

            // 调用供应商api
            SmsResult<SysSmsTemplate> smsResult = SmsUtilFactory.getSmsTemplateUtil()
                    .deleteSmsTemplateByCode(smsTemplate.getSupplierTemplateCode());
            if (!smsResult.isSuccess()) {
                LOGGER.error("【SMS】 短信模板删除失败！{}", smsResult);
                return ResultBean.fail(smsResult.getErrMsg()).setResultObject(smsResult);
            }

            getCrudService().delete(id);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @Override
    public ResultBean getById(@PathVariable("id") Long id) {
        try {
            ValidatorBean.checkNotNull(id, "请选需要查询短信模板！");
            SysSmsTemplate smsTemplate = getCrudService().get(id);

            // 调用供应商api
            SmsResult<SysSmsTemplate> smsResult = SmsUtilFactory.getSmsTemplateUtil()
                    .getSmsTemplateByCode(smsTemplate.getSupplierTemplateCode());
            if (!smsResult.isSuccess()) {
                LOGGER.error("【SMS】 短信模板查询失败！{}", smsResult);
                return ResultBean.fail(smsResult.getErrMsg()).setResultObject(smsResult);
            } else if (smsResult.isSuccess() && smsResult.getResultData() != null) {
                smsTemplate.setReason(smsResult.getResultData().getReason());
            }

            return ResultBean.success("操作成功")
                    .setResultObject(smsTemplate)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/sync-supplier-template")
    @ApiOperation("同步供应商模板信息")
    public ResultBean syncSupplierTemplate(@RequestBody SysSmsTemplate bean) {
        try {
            ValidatorBean.beginValid(bean)
                    .notNull("supplierType",bean.getSupplierType())
                    .notNull("supplierTemplateCode",bean.getSupplierTemplateCode())
                    .notNull("templateCode",bean.getTemplateCode());

            SysSmsTemplate smsTemplate = new SysSmsTemplate();
            smsTemplate.setTemplateCode(bean.getTemplateCode());

            // 唯一校验
            if (!sysSmsTemplateService.checkSysSmsTemplateOnly(smsTemplate)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                        .setErrorDetail("模板代码已存在")
                        .setErrorSolution("请重新操作")
                        .build();
            }
            // 调用供应商api
            SmsResult<SysSmsTemplate> smsResult = SmsUtilFactory.getSmsTemplateUtil()
                    .getSmsTemplateByCode(bean.getSupplierTemplateCode());
            if (!smsResult.isSuccess()) {
                LOGGER.error("【SMS】 短信模板查+询失败！{}", smsResult);
                return ResultBean.fail(smsResult.getErrMsg()).setResultObject(smsResult);
            } else if (smsResult.isSuccess() && smsResult.getResultData() == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                        .setErrorDetail("模板代码已存在")
                        .setErrorSolution("请重新操作")
                        .build();
            }
            smsTemplate = smsResult.getResultData();
            smsTemplate.setTemplateCode(bean.getTemplateCode());
            sysSmsTemplateService.insert(smsTemplate);

            return ResultBean.success("操作成功")
                    .setResultObject(smsTemplate)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
