package cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysBarcodeRule;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * @Description : 字符串解析
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2019-03-12 14:41
 * @Modify:
 **/
public final class BarcodeParseUtil {

    private BarcodeParseUtil() {
    }

    private static ImppRedis redisRes;

    /**
     * 按指定规则解析条码
     *
     * @param barcodeRuleCode 条码规则信息
     * @param barcode     条码
     * @return 条码数据
     */
    public static Map<String, String> parse(String barcodeRuleCode, String barcode) {
        if (redisRes == null) {
            redisRes = SpringContextsUtil.getRedisRes();
        }
        return parse((SysBarcodeRule) redisRes.getObject(barcodeRuleCode), barcode);
    }

    /**
     * 按指定规则解析条码
     *
     * @param barcodeRule 条码规则信息
     * @param barcode     条码
     * @return 条码数据
     */
    public static Map<String, String> parse(SysBarcodeRule barcodeRule, String barcode) {
        // 数据有效性校验
        if (barcodeRule == null || barcode == null) {
            return null;
        } else if (barcodeRule.getBarcodeRule() == null || barcodeRule.getBarcodeSeparator() == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("条码规则不完善！")
                    .build();
        }

        Long start=System.currentTimeMillis();
        //特殊分隔符处理
        barcodeRule.setBarcodeSeparator(barcodeRule.getBarcodeSeparator().replaceAll("\\\\", "\\\\\\\\"));
        barcodeRule.setBarcodeSeparator(barcodeRule.getBarcodeSeparator().replaceAll("\\|", "\\\\|"));
        barcodeRule.setBarcodeSeparator(barcodeRule.getBarcodeSeparator().replaceAll("\\.", "\\\\."));
        Long tsfgf=System.currentTimeMillis();
        System.out.println("特殊分隔符处理" + (tsfgf - start));

        // 分隔条码信息
        String[] barcodeKey = barcodeRule.getBarcodeRule().split(barcodeRule.getBarcodeSeparator());
        String[] barcodeValue = barcode.split(barcodeRule.getBarcodeSeparator());
        Long fgtm =System.currentTimeMillis();
        System.out.println("分隔条码信息" + (fgtm - tsfgf));

        // 校验
        verifyBarcode(barcodeRule,barcodeKey,barcodeValue,barcode);
        Long jy =System.currentTimeMillis();
        System.out.println("校验" + (jy - fgtm));

        // 匹配条码数据
        HashMap<String, String> barcodeMap = new HashMap<>();
        for (int i = 0; i < barcodeKey.length; i++) {
            barcodeMap.put(barcodeKey[i], barcodeValue.length > i ? barcodeValue[i] : null);
        }
        Long pp =System.currentTimeMillis();
        System.out.println("匹配条码数据" + (pp - jy));
        return barcodeMap;
    }

    /**
     * 条码校验
     *
     * @param barcodeRule    校验模式
     * @param barcodeKey   条码key
     * @param barcodeValue 条码值
     */
    private static void verifyBarcode(SysBarcodeRule barcodeRule, String[] barcodeKey, String[] barcodeValue, String barcode) {
        boolean verifyFail = false;
        if (barcodeRule.getCheckModeVal() == CommonEnumUtil.BARCODE_CHECK_MODE.SEPARATOR_NUM.getValue()) {
            // 校验分隔符的数量
            int barcodeSeparatorNum = barcode.length() - barcode.replaceAll(barcodeRule.getBarcodeSeparator(), "").length() + 1;
            int ruleSeparatorNum =barcodeRule.getBarcodeRule().length()
                    - barcodeRule.getBarcodeRule().replaceAll(barcodeRule.getBarcodeSeparator(), "").length() + 1;

            verifyFail = ruleSeparatorNum != barcodeSeparatorNum;
        } else if (barcodeRule.getCheckModeVal() == CommonEnumUtil.BARCODE_CHECK_MODE.ANALYTIC_NUM.getValue()) {
            // 校验解析后的数量是否相等
            verifyFail = barcodeKey.length != barcodeValue.length;
        }

        if (verifyFail) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("条码与规则不匹配！")
                    .build();
        }
    }

    public static void main(String[] args) {
        String str = "{\n" +
                "\t\t\"id\": \"1202112194247430144\",\n" +
                "\t\t\"organizeCode\": \"\",\n" +
                "\t\t\"isValid\": 1,\n" +
                "\t\t\"isDeleted\": 2,\n" +
                "\t\t\"createUser\": \"qianhuasheng\",\n" +
                "\t\t\"createDatetime\": \"2019-12-04 14:27:35\",\n" +
                "\t\t\"modifyUser\": \"qianhuasheng\",\n" +
                "\t\t\"modifyDatetime\": \"2019-12-04 14:27:35\",\n" +
                "\t\t\"createDateTimeStart\": null,\n" +
                "\t\t\"createDateTimeEnd\": null,\n" +
                "\t\t\"modifyDateTimeStart\": null,\n" +
                "\t\t\"modifyDateTimeEnd\": null,\n" +
                "\t\t\"orderByParam\": \"\",\n" +
                "\t\t\"ascOrDesc\": 1,\n" +
                "\t\t\"sortParamMap\": null,\n" +
                "\t\t\"name\": \"VDA生产标签打印解析二维码\",\n" +
                "\t\t\"barcodeRuleCode\": \"VDA_REPORT_SN_2D\",\n" +
                "\t\t\"barcodeRule\": \"partNo|express\",\n" +
                "\t\t\"barcodeSeparator\": \"|\",\n" +
                "\t\t\"barcodeRuleDescription\": \"VDA生产标签打印解析二维码\",\n" +
                "\t\t\"isValidVal\": 1,\n" +
                "\t\t\"checkMode\": 10,\n" +
                "\t\t\"isDeletedVal\": 2\n" +
                "\t}";

        System.out.println(parse(JsonUtilTool.decode(str,SysBarcodeRule.class),"YFRHU|"));
    }
}
