package cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.MailConfig;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import com.sun.mail.smtp.SMTPAddressFailedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.*;
import javax.mail.internet.*;
import javax.mail.util.ByteArrayDataSource;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * @Description : 邮件工具类
 * @Reference :
 * @Author : yunhaoo
 * @CreateDate : 2018-11-14 19:24
 * @Modify:
 **/
@Component
public class MailUtil {
    public static final Logger LOGGER = LoggerFactory.getLogger(MailUtil.class);

    /**
     * 开启starttls , 不然有些邮箱会报认证失败的错误，如outlook
     */
    @Value("${mail.smtp.starttls.enable:false}")
    private boolean smtpStarttlsEnabled;

    // 收件人
    private Address[] to = new Address[]{};
    private Address[] cc = new Address[]{};
    private int mailSwitch = CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue();
    private String from = "";
    private String nick = "";
    private String title = "";
    private String content = "";
    private String smtpHost = "";
    private int smtpPort = 25;
    private String contentType = "";

    private String smtpUser = "";
    private String smtpPassword = "";
    private boolean isAuthenticationSMTP = false;
    private final List<BodyPart> attachmentList = new ArrayList<>();

    private static final int MAIL_SMTP_TIMEOUT = 10000;
    private static final int MAIL_SMTP_CONNECTIONTIMEOUT = 10000;
    private static final int MAIL_SMTP_WRITETIMEOUT = 10000;

    /**
     * 初始化服务器邮箱参数
     *
     * @return 初始化
     */
    public MailUtil init() {
        try {
            // 查询是否开启邮件服务器
            this.mailSwitch = RedisCacheTool.getSysConfigIntVal(PlatformConstWords.MAIL_SWITCH,
                    CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            this.smtpHost = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_HOST);
            this.smtpPort = RedisCacheTool.getSysConfigIntVal(PlatformConstWords.MAIL_PORT);
            this.from = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_USER);
            this.nick = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_NICK);
            this.smtpUser = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_USER);
            this.smtpPassword = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_PASSWORD);
            this.isAuthenticationSMTP = true;
            this.attachmentList.clear();
            return this;
        } catch (Exception e) {
            LOGGER.error("邮箱配置初始化失败！", e);
            return new MailUtil();
        }

    }

    /**
     * 初始化服务器邮箱参数
     *
     * @param mailConfig 邮箱配置信息
     * @return 初始化
     */
    public MailUtil init(MailConfig mailConfig) {
        try {
            this.mailSwitch = mailConfig.getMailSwitch();
            this.smtpHost = mailConfig.getMailHost();
            this.smtpPort = mailConfig.getMailPort();
            this.from = mailConfig.getMailUser();
            this.nick = mailConfig.getMailNick();
            this.smtpUser = mailConfig.getMailUser();
            this.smtpPassword = mailConfig.getMailPassword();
            this.isAuthenticationSMTP = true;

            return this;
        } catch (Exception e) {
            LOGGER.error("邮箱配置初始化失败！", e);
            return new MailUtil();
        }
    }

    /**
     * 设置收件人地址
     *
     * @param nick 收件人Email地址
     */
    public void setNick(String nick) {
        this.nick = nick;
    }


    /**
     * 设置收件人地址
     *
     * @param aEmail 收件人Email地址
     */
    public void setTo(String aEmail) {
        String[] s = new String[1];
        s[0] = aEmail;
        this.to = getAddress(s);
    }

    /**
     * 设置多个收件人地址
     *
     * @param emails 收件人Email地址
     */
    public void setTo(String[] emails) {
        this.to = getAddress(emails);
    }

    /**
     * 设置抄送地址
     *
     * @param aEmail 抄送地址
     */
    public void setCC(String aEmail) {
        String[] s = new String[1];
        s[0] = aEmail;
        this.cc = getAddress(s);
    }

    /**
     * 设置多个抄送地址
     *
     * @param emails 抄送地址
     */
    public void setCC(String[] emails) {
        this.cc = getAddress(emails);
    }

    /**
     * 设置邮件主题
     *
     * @param mailTitle 邮件主题
     */
    public void setSubject(String mailTitle) {
        this.title = mailTitle;
    }

    /**
     * 设置邮件文字内容
     *
     * @param mailContent 邮件文字内容
     */
    public void setBody(String mailContent) {
        this.content = mailContent;
    }

    /**
     * 设置邮件字符类型
     *
     * @param contentType 请从CommonConstWords.MAIL_MODE_TEXT和MAIL_MODE_HTML中选择
     */
    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    /**
     * 获取邮箱地址
     *
     * @param add 邮箱地址字符串
     * @return
     */
    private Address[] getAddress(String[] add) {
        Address[] a = new Address[add.length];
        for (int i = 0; i < add.length; i++) {
            try {
                a[i] = new InternetAddress(add[i]);
            } catch (AddressException ex) {
                LOGGER.error("【" + add[i] + "】邮箱地址错误！", ex.getMessage());
            }
        }
        return a;
    }

    /**
     * 添加附件
     *
     * @param fileName       附件名
     * @param attachmentByte
     * @throws MessagingException
     */
    public void addAttachment(String fileName, byte[] attachmentByte) throws MessagingException {
        addAttachment(fileName, attachmentByte,
                FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));
    }

    /**
     * 添加附件
     *
     * @param fileName       附件名
     * @param attachmentByte
     * @param contentType
     * @throws MessagingException
     */
    public void addAttachment(String fileName, byte[] attachmentByte, String contentType) throws MessagingException {
        BodyPart attachmentBodyPart = new MimeBodyPart();
        DataSource source = new ByteArrayDataSource(attachmentByte, contentType);
        attachmentBodyPart.setDataHandler(new DataHandler(source));
        attachmentBodyPart.setFileName(fileName);
        attachmentList.add(attachmentBodyPart);
    }

    /**
     * 添加附件
     *
     * @param attachment 附件对象
     * @throws MessagingException
     */
    public void addAttachment(File attachment) throws MessagingException {
        BodyPart attachmentBodyPart = new MimeBodyPart();
        DataSource source = new FileDataSource(attachment);
        attachmentBodyPart.setDataHandler(new DataHandler(source));
        attachmentBodyPart.setFileName(attachment.getName());
        attachmentList.add(attachmentBodyPart);
    }

    public void cleanAttachmentList() {
        attachmentList.clear();
    }

    /**
     * 发送邮件
     */
    public void send() {
        try {
            // 是否开启邮箱配置
            if (this.mailSwitch != CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                return;
            }
            Properties server = new Properties();
            server.put("mail.smtp.port", String.valueOf(this.smtpPort));
            server.put("mail.smtp.host", this.smtpHost);
            //设置邮件的默认的超时时间
            server.put("mail.smtp.timeout", MAIL_SMTP_TIMEOUT);
            server.put("mail.smtp.connectiontimeout", MAIL_SMTP_CONNECTIONTIMEOUT);
            server.put("mail.smtp.writetimeout", MAIL_SMTP_WRITETIMEOUT);

            if (this.isAuthenticationSMTP) {
                server.put("mail.smtp.auth", "true");
            }
            if (smtpStarttlsEnabled) {
                server.put("mail.smtp.starttls.enable", "true");

//                server.put("mail.smtp.ssl.protocols","TLSv1.2");
//                server.put("mail.debug","true"); //开启mail debug日志
//                server.put("mail.smtp.ssl.protocols","TLSv1.2");
//                server.put("mail.debug","true");
//                server.put("mail.smtp.auth.mechanisms","NTLM");
//                server.put("mail.smtp.auth.mechanisms","PLAIN");
//                server.put("mail.smtp.auth.mechanisms","DIGEST-MD5");
            }

            Session conn = Session.getInstance(server, null);

            MimeMessage msg = new MimeMessage(conn);
            if (nick != null && !"".equals(nick)) {
                nick = MimeUtility.encodeText(nick, MimeUtility.mimeCharset("gb2312"), null);
                msg.setSubject(nick);
                msg.setFrom(new InternetAddress(nick + " <" + from + ">"));
            } else {
                msg.setFrom(new InternetAddress(this.from));
            }
            // 收件人
            if (this.to != null) {
                msg.setRecipients(Message.RecipientType.TO, this.to);
            }
            // 抄送
            if (this.cc != null) {
                msg.setRecipients(Message.RecipientType.CC, this.cc);
            }

            msg.setSubject(MimeUtility.encodeText(this.title, MimeUtility.mimeCharset("gb2312"), null));
            msg.setSubject(this.title);

            // 消息主体内容 处理
            this.content = this.content == null ? "" : this.content;

            // 判断是否存在附件
            if (!attachmentList.isEmpty()) {
                // 添加邮件的各个部分内容，包括文本内容和附件
                Multipart multipart = new MimeMultipart();

                // 添加邮件正文
                BodyPart contentPart = new MimeBodyPart();
                contentPart.setContent(content, this.contentType);
                multipart.addBodyPart(contentPart);

                for (BodyPart bodyPart : attachmentList) {
                    multipart.addBodyPart(bodyPart);
                }
                msg.setContent(multipart);
            } else {
                msg.setContent(this.content, this.contentType);
            }

            msg.saveChanges();
            if (this.isAuthenticationSMTP) {
                Transport transport = conn.getTransport("smtp");
                transport.connect(this.smtpHost, this.smtpUser, this.smtpPassword);
                transport.sendMessage(msg, msg.getAllRecipients());
                transport.close();
            } else {
                Transport.send(msg, msg.getAllRecipients());
            }
            LOGGER.info("邮件发送成功");

        } catch (SMTPAddressFailedException e) {
            LOGGER.error("邮件发送异常", e);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_RECIPIENT_NOT_EXIST.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_RECIPIENT_NOT_EXIST.getDescription())
                    .setErrorSolution("请重新设置收件人")
                    .build();
        } catch (AuthenticationFailedException e) {
            LOGGER.error("邮件发送异常", e);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_LOGIN_PASS_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_LOGIN_PASS_ERROR.getDescription())
                    .setErrorSolution("请重新设置账号或密码")
                    .build();
        } catch (MessagingException e) {
            LOGGER.error("邮件发送异常", e);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getDescription())
                    .setErrorSolution("请重新设置邮件服务器地址或端口")
                    .build();
        } catch (UnsupportedEncodingException e) {
            LOGGER.error("邮件发送异常", e);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getDescription())
                    .setErrorSolution("请重新设置邮件服务器地址或端口")
                    .build();
        } catch (Exception e) {
            LOGGER.error("邮件发送异常", e);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getDescription())
                    .setErrorSolution("邮件发送异常：%s", e.getMessage())
                    .build();
        }
    }

    public static void main(String[] args) throws IOException, MessagingException {
        MailConfig mailConfig = new MailConfig();
        mailConfig.setMailSwitch(1);
        mailConfig.setMailHost("smtphm.qiye.163.com");
        mailConfig.setMailPort(25);
        mailConfig.setMailNick("impp-test@estsh.com");
        mailConfig.setMailUser("impp-test@estsh.com");
        mailConfig.setMailPassword("Aa135790");
        MailUtil mailUtil = new MailUtil();
        mailUtil.init(mailConfig);
        mailUtil.setNick("SFP验证码");
        mailUtil.setTo("castle.zang@estsh.com");
//        mailUtil.setTo("ying.he.zang@estsh.com");
        mailUtil.setContentType(ImppEnumUtil.MESSAGE_TYPE_CONTENT.TEXT.getDescription());
        mailUtil.setSubject("SFP验证码");
        mailUtil.setBody(
                "供应商账号(登录名称),您好!\n" +
                "\n" +
                "您登录账号“供应商账号（登录名称）”修改密码所需的验证码为: “XXXXX“\n"+
                "\n" +
                "您会收到这封自动产生的邮件，是由于有人在供应商端试图通过网络设备修改您的账户密码，且提供了正确的账户名称与邮箱。\n" +
                "验证码是修改密码所必需的。没有人能够不访问这封电子邮件就修改您的账户密码。\n" +
                "\n" +
                "\n" +
                "\n" +
                "\n" +
                "祝您愉快，\n" +
                "SFP团队");
        mailUtil.send();

    }
}
